unit untCadEquip;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, DB, Mask, DBCtrls, DBClient;

type
  TfrmCadEquip = class(TForm)
    pnlTop: TPanel;
    edtBuscaCodigo: TEdit;
    Label1: TLabel;
    btnInserir: TBitBtn;
    btnAlterar: TBitBtn;
    btnCancelar: TBitBtn;
    btnSalvar: TBitBtn;
    dsrDataSource: TDataSource;
    btnExcluir: TBitBtn;
    pnlMain: TPanel;
    Label2: TLabel;
    dbeCodigo: TDBEdit;
    Label3: TLabel;
    dbeDtCadastro: TDBEdit;
    Label4: TLabel;
    dbePatrimonio: TDBEdit;
    Label5: TLabel;
    dbeLocalidade: TDBEdit;
    Label6: TLabel;
    dbeModelo: TDBEdit;
    Label7: TLabel;
    dbeMarca: TDBEdit;
    Label8: TLabel;
    dbeDescricao: TDBEdit;
    btnSalvarOff: TBitBtn;
    dbrSituacao: TDBRadioGroup;
    procedure btnInserirClick(Sender: TObject);
    procedure btnAlterarClick(Sender: TObject);
    procedure btnExcluirClick(Sender: TObject);
    procedure btnCancelarClick(Sender: TObject);
    procedure btnSalvarClick(Sender: TObject);
    procedure edtBuscaCodigoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure btnSalvarOffClick(Sender: TObject);
  private
    FDataSetEquip: TDataSet;
    FModoOff: Boolean;
    procedure SetDataSetEquip(const Value: TDataSet);
    procedure SetModoOff(const Value: Boolean);
    { Private declarations }
  public
    { Public declarations }
    property DataSetEquip: TDataSet read FDataSetEquip write SetDataSetEquip;
    property ModoOff: Boolean read FModoOff write SetModoOff;
  end;

var
  frmCadEquip: TfrmCadEquip;

const
  FileDadosOff: String = 'Dados_Offline.xml';

implementation

uses untDM;

{$R *.dfm}

procedure TfrmCadEquip.btnInserirClick(Sender: TObject);
begin
  //abre o DataSet e chama o mtodo de insero
  dsrDataSource.DataSet.Open;
  dsrDataSource.DataSet.Append;

  //atribui a data atual para o campo Data Cadastro
  dsrDataSource.DataSet.FieldByName('DATA_CADASTRO').AsDateTime:= Now;

  //foca no Cdigo
  dbeCodigo.SetFocus;
end;

procedure TfrmCadEquip.btnAlterarClick(Sender: TObject);
begin
  //verifica se h registro a ser alterado
  if not dsrDataSource.DataSet.IsEmpty then
  begin
    //chama o mtodo de edio
    dsrDataSource.Edit;

    //desabilita o campo Cdigo
    dbeCodigo.Enabled:= False;

    //foca no Cdigo
    dbePatrimonio.SetFocus;
  end;
end;

procedure TfrmCadEquip.btnExcluirClick(Sender: TObject);
begin
  //verifica se h registro a ser excludo
  if not dsrDataSource.DataSet.IsEmpty then
  begin
  
    //emite mensagem de confirmao
    if Application.MessageBox('Deseja confirmar excluso deste registro?',
      'Excluso', MB_YESNO+MB_ICONQUESTION) = mrYes then
    begin

      //chama o mtodo de excluso
      dsrDataSource.DataSet.Delete;

      //chama ApplyUpdates do ClientDataSet -> declarar unit DBClient em uses
      TClientDataSet(dsrDataSource.DataSet).ApplyUpdates(0);
    end;
  end;
end;

procedure TfrmCadEquip.btnCancelarClick(Sender: TObject);
begin
  //verifica se o DataSet est em Insero ou Edio para chamar o mtodo Cancel
  if dsrDataSource.DataSet.State in [dsInsert, dsEdit] then
    dsrDataSource.DataSet.Cancel;
end;

procedure TfrmCadEquip.btnSalvarClick(Sender: TObject);
begin
  //verifica se o DataSet est em Insero ou Edio
  if dsrDataSource.DataSet.State in [dsInsert, dsEdit] then
  begin

    //salva o registro em memria e na base de dados e emite mensagem de sucesso
    dsrDataSource.DataSet.Post;

    //s chama o mtodo ApplyUpdates se estiver configurado para salvar os dados no banco de dados
    if not FModoOff then
      TClientDataSet(dsrDataSource.DataSet).ApplyUpdates(0);
      
    ShowMessage('Registro salvo com sucesso!');
  end;
end;

procedure TfrmCadEquip.edtBuscaCodigoKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  //verifica se a tecla Enter foi pressionada e se h texto na caixa edtBuscaCodigo
  if (Key = vk_return) and (Trim(edtBuscaCodigo.Text) <> EmptyStr) then
  begin
    //passa o parmetro para o ClientDataSet para que ele busque o registro especificado
    dsrDataSource.DataSet.Close;
    TClientDataSet(dsrDataSource.DataSet).Params[0].AsString:= Trim(edtBuscaCodigo.Text);
    dsrDataSource.DataSet.Open;
  end;
end;

procedure TfrmCadEquip.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //se for Modo Offline
  if ModoOff then
  begin
    //verifica se h dados em memria
    if not dsrDataSource.DataSet.IsEmpty then
    begin
      //emite mensagem de aviso e salva os dados em um arquivo xml
      Application.MessageBox('Os dados em memria sero salvos em arquivo', 'Ateno', MB_ICONINFORMATION);
      TClientDataSet(dsrDataSource.DataSet).SaveToFile(FileDadosOff, dfXML);
      TClientDataSet(dsrDataSource.DataSet).EmptyDataSet;
    end;
  end
  else
    //ao trabalhar com banco de dados, garante que o DataSet sempre seja fechado ao encerrar a tela
    dsrDataSource.DataSet.Close;
end;

procedure TfrmCadEquip.SetDataSetEquip(const Value: TDataSet);
begin
  FDataSetEquip := Value;

  //atribui  propriedade DataSet do DataSource o DataSet passado  property
  dsrDataSource.DataSet:= FDataSetEquip;
end;

procedure TfrmCadEquip.FormShow(Sender: TObject);
begin
  //garante que o DataSet sempre inicie fechado
  dsrDataSource.DataSet.Close;

  //o boto SalvarOff s fica disponvel caso no esteja em ModoOff e
  //existam dados em arquivo a serem salvos no banco 
  btnSalvarOff.Visible:= not(ModoOff) and FileExists(FileDadosOff);
end;

procedure TfrmCadEquip.SetModoOff(const Value: Boolean);
begin
  FModoOff := Value;

  //para o modo Off, so desabilitadas algumas opes do sistema
  if FModoOff then
  begin
    edtBuscaCodigo.Enabled:= False;
    btnAlterar.Enabled:= False;
    btnExcluir.Enabled:= False;
  end;
end;

procedure TfrmCadEquip.btnSalvarOffClick(Sender: TObject);
var
  i: Integer;
begin
  //carrega dados do arquivo em memria
  DM.cdsTempEquip.LoadFromFile(FileDadosOff);

  //varre toda a estrutura
  DM.cdsTempEquip.Open;
  DM.cdsTempEquip.First;
  while not DM.cdsTempEquip.Eof do
  begin
    dsrDataSource.DataSet.Open;
    dsrDataSource.DataSet.Append;

    //como as estruturas de ambos ClientDataSets j esto igualadas, apenas atribuimos o valor
    //de um para o outro
    for i:= 0 to Pred(DM.cdsTempEquip.FieldCount) do
      dsrDataSource.DataSet.Fields[i].Value:= DM.cdsTempEquip.Fields[i].Value;

    //salva o registro no banco de dados  
    dsrDataSource.DataSet.Post;
    TClientDataSet(dsrDataSource.DataSet).ApplyUpdates(0);

    DM.cdsTempEquip.Next;
  end;

  //esvazia a estrutura em memria, deleta o arquivo xml e oculta o boto SalvaOff
  DM.cdsTempEquip.EmptyDataSet;
  DeleteFile(FileDadosOff);
  btnSalvarOff.Visible:= False;
end;

end.

